<?php
/**
 * @package PDO_For_Wordpress
 * @version $Id$
 * @author	Justin Adie, rathercurious.net
 */
 
 /**
  *	this class inherits from wpdb
  *
  * the database connection mechanism is handed off by createDBOObject() to the secondary abstraction class
  * various other methods override the wpdb versions.
  * Thanks to Ulf Ninow for the suggestion on inheritance.  this will make life
  * easier to maintain the package for upgrades and extensions for other database backends.
  */


//call the PDO engine
require_once PDODIR.'PDOEngine.php';

/*
 * this definition is used by EZSQL
 */
if (!defined('SAVEQUERIES')){
	define ('SAVEQUERIES', true);
}

/*
 * PDO_DEBUG set this to true to create a list of queries in your database directory that can be used to debug
 */
if(!defined('PDO_DEBUG')){
	define('PDO_DEBUG', FALSE);
}

//provide a reference for the native wpdb class
if ( ! isset($wpdb) ){
	global $wpdb;
	require_once ABSPATH.'wp-includes/wp-db.php';
	unset($wpdb);
}

class pdo_db extends wpdb {

	/**
	*	variables added for the sqlite port
	*
	*	@var $engine mixed - holds a reference to the main query engine
	*	@var $queryType string - holds the type of query, as set by determineQueryType
	*/
	
	var $engine;
	var $queryType;

	/**
	 * Connects to the database server with params needed to create a viable dsn
	 *
	 * @param  $dbuser string
	 * @param  $dbpassword string
	 * @param  $dbname string
	 * @param  $dbhost string
	 */
	function pdo_db($dbuser=null, $dbpassword=null, $dbname, $dbhost, $dbType) {
		$this->dbType = $dbType;
		$this->dbuser = $dbuser;
		$this->dbpassword = $dbpassword;
		$this->dbname = $dbname;
		$this->dbhost = $dbhost;
		register_shutdown_function(array(&$this, "__destruct"));

		if ( defined('DB_CHARSET') )
			$this->charset = DB_CHARSET;

		if ( defined('DB_COLLATE') )
			$this->collate = DB_COLLATE;
		
		$this->createDBObject();
	
		if ($this->dbh->isError) {
			$this->bail($this->dbh->getErrorMessage());
		}
		
	}
	
	
	/**
	*	sets the database connection object
	*
	*	this function intermediates within the native
	*	db abstraction class for wordpress.
	*	and allows a connection type object to be created across
	*	the supported database types that have PDO drivers and rewrite scripts provided
	*	no variable is returned but the dbh property is set
	*/
	function createDBObject(){	
		$this->dbh = NEW PDO_Engine(array($this->dbType, $this->dbuser, $this->dbpassword, $this->dbname, $this->dbhost));
		if ($this->dbh === false || $this->dbh->isError){
			$this->bail($this->dbh->getErrorMessage());
		}
	}

	/**
	 *	method to escape the string for use in mysql databases.
	 *
	 *	this is left intact rather than dummying out.  there is a performance hit in doing so
	 *	but trying to dummy it screws with some other WP functionality 
	 *	similarly, using mysql_escape_string() (which you would think would be logical) screws things up to.
	 *	basically does not honour line terminators.
	 *
	 *	@param $string	string	the variable to be escaped
	 *	@return	string	escaped variable
	 */
	function escape($string) {
		return addslashes($string);
	}
	
	/**
     *	sends a query through the abstraction engine 
	 *
	 *	this is largely the same as the original query function
	 *	within wordpress save that it executes via an engine
	 *	rather than directly
	 *
	 *	@param $query string  - the query to be executed
	 *	@return integer/bool - depends on the queryType
	 */
	function query($query) {
		
		// filter the query, if filters are available
		// NOTE: some queries are made before the plugins have been loaded, and thus cannot be filtered with this method
		if ( function_exists('apply_filters') )
			$query = apply_filters('query', $query);

		// initialise return
		$return_val = 0;
		$this->flush();

		// Log how the function was called
		$this->func_call = "\$db->query(\"$query\")";

		// Keep track of the last query for debug..
		$this->last_query = $query;

		
		if (SAVEQUERIES)
			$this->timer_start();
	
		//run the query through the abstraction engine
		
		// and this, basically, is where all the heavy lifting is done!  
		$this->result = $this->dbh->query($query);
		
		++$this->num_queries;

		if (SAVEQUERIES)
			$this->queries[] = array( $query, $this->timer_stop(), $this->get_caller() );
		
		// If there is an error then take note of it..
		//	this has been changed to reference the PDO_Engine object which has more
		//	advanced error handling and reporting
		if ( $this->dbh->isError === true ) {
			if (defined('WP_INSTALLING') && WP_INSTALLING === true){
			} else {
				$this->bail($this->dbh->getErrorMessage());
			}
		}
		
		$this->insert_id = $this->dbh->getInsertID();
		$this->rows_affected = $this->dbh->getAffectedRows();
		$this->col_info = $this->dbh->getColumns();
		$this->last_result = $this->dbh->getQueryResults();
		$this->num_rows = $this->dbh->getNumRows();
		$return_val = $this->dbh->getReturnValue();
		//print_r($return_val);
		return $return_val;
	}

	
	/**
	 * Checks wether of not the database version is high enough to support the features WordPress uses
	 * @global $wp_version
	 */
	function check_database_version(){
		global $wp_version;
		// Make sure the server has MySQL 4.0
		switch ($this->dbType){
			case "mysql":
				// Make sure the server has MySQL 4.0
				if ( version_compare($this->db_version(), '4.0.0', '<') ) {
					return new WP_Error('database_version',sprintf(__('<strong>ERROR</strong>: WordPress %s requires MySQL 4.0.0 or higher'), $wp_version));
				}
				break;
			case "sqlite":
				return true;
				break;
		}
	}

	/**
	 * abstract db_version from ezsql
	 */
	function db_version(){
		switch ($this->dbType) {
			case 'sqlite':
				return 5.0;
				break;
			case 'mysql':
				return 5.0;
				break;
			default:
				return false;
		}
	}

	/**
	 * This function is called when WordPress is generating the table schema to determine wether or not the current database
	 * supports or needs the collation statements.
	 */
	function supports_collation(){
		if ($this->dbType == "mysql"){
			return ( version_compare(mysql_get_server_info(), '4.1.0', '>=') );
		} else {
			return false;
		}
	}
}

/**
 *	let's go and create the database object.
 *
 *	note that this is changed from wordpress standard to include the DB_TYPE in the parameters
 */
if ( ! isset($wpdb) ){
	global $wpdb;
	$wpdb = new pdo_db(DB_USER, DB_PASSWORD, DB_NAME, DB_HOST, DB_TYPE);
}

?>